/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Js component                                                         **
**                                                                       **
**	Copyright(C) 2018 - 2020 by George Malamas                           **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: June 14, 2020                                             **
**                                                                       **
**	Release number: 9                                                    **
**                                                                       **
***************************************************************************
*/
// This our js component, in which is implemented our main observer, which is always registered if there is
// at least one browser window.
// Our observer is used in appVer >= 3 and has three purposes:
// 1. In appVer 3-19, gets the restart request notification and help us to patch "on the fly" the
//    embedded restart dialog of browser. Details for this can be found in our main code file.
// 2. In appVer >= 20, gets any normal restart request notification, cancels it and helps us take the
//    control, letting the user decide what to do, in accordance the restart options we offer.
// 3. In appVer >= 20, gets any normal exit request notification and depending on conditions, does the following:
//    In private browsing mode, cancels the request and passes the control to us.
//    In non-private browsing mode, helps us to perform any pending actions we do in some cases on app exiting
//    or restarting. 
//    All the details about, can be found in our main code file.
// Our js component also, is used as a "Per session" global data storage.

"use strict";

const {classes: Cc, Constructor: CC, interfaces: Ci, results: Cr, utils: Cu} = Components;
const appVersion = parseFloat(Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo).version);
var cs = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);

// Factory object.
var RestartRequestObserverServiceFactory = {
	instance: null,
	createInstance: function RFSF_createInstance(outer, iid) {
		if(outer != null) {
			throw Cr.NS_ERROR_NO_AGGREGATION;
		};
		return (this.instance == null) ? this.instance = new RestartRequestObserver() : this.instance;
	}
};

// Global variables - "Per session" data storage.
const propStoreInternal = {
	observerIsRegistered: false, // Our "Restart request observer", appVer 3+.
	appHasStarted: false, // appVer >= 3.
	weAreQuitting: false, // appVer >= 3, utilized only by appVer >=20.
	internalRequest: false, // appVer >= 20, used as identifier by our observer to distinct internal/external requests.
	restartFlag: false, // Used as identifier of restart dialog loading in appVer 3-19.
	allowCheck_prvBrowsingAutostart: true, // appVer >= 20.
	userSettingsRestored: false, // appVer >= 20, used to do the job only once, if there are > 1 windows.
	weAreRestarting: false, // appVer >= 20.
	forceRestarted: false, // appVer >= 4.
	allowSpecials: true, // appVer >= 4. Specials = "force restart", dev prefs.
	versionChecked: false, // Flag to check our version only once per session, appVer >= 3.
	restartRequested: false, // Flag to take into account on "quit-application-granted", to determine if the browser is going to restart or quit, appVer >= 4.
	startedWithNoRemote: false, // appVer >= 3.5.
	clearMozNoRemote: false, // "MOZ_NO_REMOTE" environment variable, appVer >= 3.5.
};

// Class constructor.
function RestartRequestObserver() {
	this.wrappedJSObject = this;
	this.onApplicationStartup();
	if(appVersion >= 20) {
		this.getPrvBrowsingAutostart(true);
	};
};

// Class definition.
RestartRequestObserver.prototype = {
	classDescription: "Restart manager - Restart Request Observer",
	contractID: "@restart_manager/chania.gr;1",
	classID: Components.ID("{d6ecd398-dad8-4f6f-9edb-95822afaf179}"),
	_xpcom_factory: RestartRequestObserverServiceFactory,
	_xpcom_categories: [{
						category: "profile-after-change", service: true}, // Gecko >= 2.0
						{category: "app-startup", service: true // Gecko = 1.9.x
						}],

	obsSrvc:      Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService),
	prefBranch:   Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).QueryInterface(Ci.nsIPrefBranch),
	promptSrvc:   Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService),
	rmPrefBranch: Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("extensions.restartmanager."),
	brPrefBranch: Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("browser."),
	winMed:       Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator),
	nsITimer:     CC("@mozilla.org/timer;1", "nsITimer", "initWithCallback"),
	nsIEnv:       Cc["@mozilla.org/process/environment;1"].getService(Ci.nsIEnvironment),

// Generic getter/setter of "propStoreInternal" data, for external requesters.
	getSet: function(prop, gsv) {
		var aName = null;
		var gs = {
			get aProp() {
				return propStoreInternal[aName];
			},
			set aProp(val) {
				propStoreInternal[aName] = val;
				aName = null;
			}
		};
		if(prop in propStoreInternal) {
			aName = prop;
			switch(gsv) {
				case "getV": // get value.
					var aProp = gs.aProp;
					aName = null;
					return aProp;
				case "setT": // set true.
					gs.aProp = true;
					return;
				case "setF": // set false.
					gs.aProp = false;
					return;
			};
		};
	},

	codeMonitoring: function() {
		var _this = this;
		var getter = {
			get cdm() {
				return _this.rmPrefBranch.getBoolPref("codeMonEnabled");
			}
		};
		return getter.cdm;
	},

// Component implementation.
  	observe: function RestMan_RestartRequestObserver(aSubject, aTopic, aData) {
		if(this.codeMonitoring() && aTopic !== "domwindowclosed") {
			if(propStoreInternal.forceRestarted && (aTopic == "sessionstore-state-read" || aTopic == "sessionstore-windows-restored")) {
				var subject = aSubject ? aSubject.toString() : aSubject;
				new this.nsITimer(function() { // The delay is required, to not break our "force restart" process.
					return this.promptSrvc.alert(null, "Observe:", "Subject: " + subject + "\nTopic: " + aTopic + "\nData: " + aData +
						"\n--------------------" + "\nStarted with \"-no-remote\": " + propStoreInternal.startedWithNoRemote);
				}.bind(this), 850, Ci.nsITimer.TYPE_ONE_SHOT);
			} else {
				var onQuitMsg = "";
				if((aTopic == "quit-application-requested") || (aTopic == "quit-application-granted") || (aTopic == "quit-application")) {
					onQuitMsg = "\nClear \"MOZ_NO_REMOTE\": " + propStoreInternal.clearMozNoRemote;
				};
				this.promptSrvc.alert(null, "Observe:", "Subject: " + aSubject + "\nTopic: " + aTopic + "\nData: " + aData +
					"\n--------------------" + "\nStarted with \"-no-remote\": " + propStoreInternal.startedWithNoRemote + onQuitMsg);
			};
		};
		var und = undefined;
		var currentBrWin = this.winMed.getMostRecentWindow("navigator:browser");
		if((aSubject instanceof Ci.nsISupportsPRBool) && aSubject.data) {
			return; // If the quit request has been canceled by someone (..?).
		};
		switch(aTopic) {
			case "sessionstore-state-read":
				if(aSubject) { // [xpconnect wrapped nsISupports].
					try {
						aSubject.QueryInterface(Ci.nsISupportsString).data = 
						(aSubject.QueryInterface(Ci.nsISupportsString).data).replace(/"state":"stopped"/g, "\"state\":\"running\"");
					} catch(ex) {
						Cu.reportError("RestMan - Error report: " + ex);
					};
					this.obsSrvc.removeObserver(this, aTopic);
				};
				break;
			case "sessionstore-windows-restored":
				this.observerRegister(aTopic);
				this.obsSrvc.removeObserver(this, aTopic);
				propStoreInternal.appHasStarted = true;
				break;
			case "quit-application-requested":
				if(aData == "lastwindow") { // On closing last window.
					propStoreInternal.restartRequested = false;
					return;
				};
				var detectOpenedDialogs = function(root) {
					function cancelRequest() {
						aSubject.QueryInterface(Ci.nsISupportsPRBool);
						aSubject.data = true;
					};
					if(currentBrWin._RestMan.temps.rejectRequest) {
						cancelRequest();
						return true;
					};
					if(currentBrWin._RestMan.locals.detectOpenDlgs && currentBrWin._RestMan.dialogDetector() &&
					   (!currentBrWin._RestMan.temps.lastDetectedDlg || (currentBrWin._RestMan.temps.openedDialog !==
					   currentBrWin._RestMan.temps.lastDetectedDlg))) {
						cancelRequest(); // Cancel external restart (appVer < 20 & >= 3) or exit (appVer 3+) request.
						currentBrWin._RestMan.temps.rejectRequest = true;
						currentBrWin._RestMan.temps.lastDetectedDlg = currentBrWin._RestMan.temps.openedDialog;
						currentBrWin._RestMan.temps.openedDialog.focus();
						currentBrWin._RestMan.temps.openedDialog.addEventListener("unload", function onUnload() {
							this.removeEventListener("unload", onUnload, false);
							currentBrWin._RestMan.temps.lastDetectedDlg = null;
						}, false);
						currentBrWin.setTimeout(function(cBrWin) { // "hasFocus()" is not that reliable, especially on Linux..
							try { // Unlikely to happen due to this tiny timeframe, but if the target has been closed, we'll
							// be broken in appVer < 15 (TypeError). We'll be broken too if "instantDlg()" is called,
							// resulting in "rejectRequest" to remain "true".
								if(!cBrWin._RestMan.temps.openedDialog.document.hasFocus()) {
									cBrWin._RestMan.instantDlg(true); // Comment in "instantDlg()", "restmanMain.js".
								} else {
									cBrWin._RestMan.blinker(true, 0);
								};
							} catch(ex) {
								if(ex) {
									cBrWin._RestMan.temps.rejectRequest = false;
									cBrWin._RestMan.temps.openedDialog = null;

									// Repeat the same request as before exception: "aData ? restart : exit".
									var cancelQuit = Cc["@mozilla.org/supports-PRBool;1"].createInstance(Ci.nsISupportsPRBool);
									var appStartup = Cc["@mozilla.org/toolkit/app-startup;1"].getService(Ci.nsIAppStartup);
									root.obsSrvc.notifyObservers(cancelQuit, aTopic, aData);
									if(appVersion < 20 && !cancelQuit.data && aData) {
										appStartup.quit(Ci.nsIAppStartup.eAttemptQuit | Ci.nsIAppStartup.eRestart);
									};
									if(!aData && !cancelQuit.data) {
										appStartup.quit(Ci.nsIAppStartup.eAttemptQuit);
									};
								};
							};
						}, 100, currentBrWin);
						return true;
					};
					return false;
				};
				if(aData == "restart") { // On internal/external restart request.
					propStoreInternal.restartRequested = true;
					try {
						if(appVersion >= 20) {
							if(propStoreInternal.internalRequest) { // On internal restart request.
								propStoreInternal.internalRequest = false;
								propStoreInternal.weAreRestarting = true;
								return;
							};
							var recentWin = this.winMed.getMostRecentWindow(null);
							if(recentWin.document.getElementById("BrowserPreferences") &&
							  (recentWin.document.documentElement.getAttribute("lastSelected") == "panePrivacy")) {
								this.observerUnregister("ppbm");
								propStoreInternal.weAreRestarting = true;
								return; // On enabling/disabling permanent private browsing mode.
							};
							if(detectOpenedDialogs(this)) {
								return;
							};
							if((appVersion >= 27 && currentBrWin.content.document.documentURI.startsWith("about:support")) ||
							   (appVersion >= 28 && currentBrWin.content.document.documentURI.startsWith("about:profiles"))) {
								return; // From our v2.05, we take the control in these cases, but we let this as it was,
							};			// just in case something goes wrong..
							aSubject.QueryInterface(Ci.nsISupportsPRBool);
							aSubject.data = true; // Cancel external restart request.
							currentBrWin._RestMan._RestManOnRestartRequest("restart", "restart", "rro"); // Get control.
						} else if(appVersion < 20) { // AppVer 3-19.
							if(detectOpenedDialogs(this)) {
								return;
							};
							if(this.brPrefBranch.getBoolPref("warnOnRestart") &&
							  (currentBrWin._RestMan.winTabCounter("getAllTabCountRRO", und, "rro") > 1)) {
								if(this.brPrefBranch.getIntPref("startup.page") == 3) {
									this.brPrefBranch.setIntPref("startup.page", 1);
									var root = this;
									new this.nsITimer(function() {
										root.brPrefBranch.setIntPref("startup.page", 3);
									}, 250, Ci.nsITimer.TYPE_ONE_SHOT);
								};
								if(appVersion > 3 && !Cc["@mozilla.org/privatebrowsing;1"].
									getService(Ci.nsIPrivateBrowsingService).privateBrowsingEnabled) {
									// No dialog in private browsing mode or when there is only 1 tab.
									propStoreInternal.restartFlag = true;
								} else if(appVersion < 3.5 ) { // AppVer 3.
									// No dialog when there is only 1 tab.
									propStoreInternal.restartFlag = true;
								};
							};
						};
					} catch(ex) {
						Cu.reportError("RestMan - Error report: " + ex);
					};
				} else if(!aData) { // On "File --> Exit".
					propStoreInternal.restartRequested = false;
					try {
						if(detectOpenedDialogs(this)) {
							return;
						};
					} catch(ex) {
						Cu.reportError("RestMan - Error report: " + ex);
					};
					if(appVersion >= 20) {
						if(propStoreInternal.internalRequest) { // On internal quit request.
							propStoreInternal.internalRequest = false;
							return;
						};
						var prvStatus = currentBrWin._RestMan.winTabCounter("getPrvStatusRRO", und, "rro");
						// [allWinPrivate, allTabsPrivate, nPrvNPndTabCnt, allTabCount, allPinnedTabs].
						try {
							if(prvStatus[0] || prvStatus[1] || (prvStatus[2] == 0 && (prvStatus[3] - prvStatus[4]) > 1)) {
								aSubject.QueryInterface(Ci.nsISupportsPRBool);
								aSubject.data = true; // Cancel app exit request.
								currentBrWin._RestMan._RestManOnRestartRequest("exit", "handleExit", "rro");
							} else {
								if(currentBrWin._RestMan.locals.alwaysWrnOnExit) {
									if(currentBrWin._RestMan.onQuitRequest("checkShowWarn", und, "rro") == "cancelReq") {
										aSubject.QueryInterface(Ci.nsISupportsPRBool);
										aSubject.data = true; // Cancel app exit request.
									};
								} else {
									currentBrWin._RestMan.onQuitRequest(und, und, "rro");
								};
							};
						} catch(ex) {
							Cu.reportError("RestMan - Error report: " + ex);
						};
					};
				};
				break;
			case "domwindowclosed":
				if(!currentBrWin) {
					propStoreInternal.restartRequested = false;
					this.observerUnregister(aTopic);
				};
				break;
			case "quit-application-granted": // appVersion >= 3.6.
			case "quit-application": // appVersion < 3.6 & >= 3.
				propStoreInternal.weAreQuitting = true;
				if(appVersion >= 20) {
					this.restoreSettingsOnQuit();
				};
				this.observerUnregister(aTopic);
				this.obsSrvc.removeObserver(this, aTopic);
				if((appVersion >= 4) && !propStoreInternal.restartRequested && this.rmPrefBranch.getBoolPref("invalidate_caches")) {
					this.rmPrefBranch.setBoolPref("invalidate_caches", false);
					this.rmPrefBranch.setBoolPref("invalCaches_highlight", false);
					if(this.prefBranch.getBranch("nglayout.").getPrefType("debug.disable_xul_fastload") !== this.prefBranch.PREF_INVALID) {
						this.prefBranch.getBranch("nglayout.").clearUserPref("debug.disable_xul_fastload");
					};
					if(this.prefBranch.getBranch("nglayout.").getPrefType("debug.disable_xul_cache") !== this.prefBranch.PREF_INVALID) {
						this.prefBranch.getBranch("nglayout.").clearUserPref("debug.disable_xul_cache");
					};
				};
				if(propStoreInternal.clearMozNoRemote) {
					this.nsIEnv.set("MOZ_NO_REMOTE", "");
				};
				break;
		};
	},

	observerRegister: function(aTopic) {
		if(!propStoreInternal.observerIsRegistered && !propStoreInternal.weAreQuitting) {
			this.obsSrvc.addObserver(this, "quit-application-requested", false);
			this.obsSrvc.addObserver(this, "domwindowclosed", false); // We can't use "browser-lastwindow-close-granted" (preferred)
			// for 2 reasons: 1.- In case of "onbeforeunload" dialog, due to a bug (obviously) in all pm versions 3.5-28, this
			// notification is sent before the dialog is handled by the user, so we can't know if the window will be closed ("Leave Page")
			// or not ("Stay on Page"). 2.- It is not supported by pm older than v3.6.
			propStoreInternal.observerIsRegistered = true;
			if(this.codeMonitoring()) {
				var reg = aTopic ? "\""+aTopic+"\" (app startup)" : "1st browser window opening";
				if(propStoreInternal.forceRestarted) {
					propStoreInternal.forceRestarted = false;
					new this.nsITimer(function() { // The delay is required, to not break our "force restart" process.
						return this.promptSrvc.alert(null, "Restart Request Observer:", "Registered");
					}.bind(this), 1000, Ci.nsITimer.TYPE_ONE_SHOT);
				} else {
					this.promptSrvc.alert(null, "Restart Request Observer:", "Registered" + "\nBy: " + reg);
				};
			};
		};
	},

	observerUnregister: function(aParam) {
		if(propStoreInternal.observerIsRegistered) {
			this.obsSrvc.removeObserver(this, "quit-application-requested");
			this.obsSrvc.removeObserver(this, "domwindowclosed");
			propStoreInternal.observerIsRegistered = false;
			if(this.codeMonitoring()) {
				var unreg = null;
				switch(aParam) {
					case "domwindowclosed":
						unreg = "\""+aParam+"\" (last browser window)";
						break;
					case "quit-application-granted": // appVersion >= 3.6.
					case "quit-application": // appVersion < 3.6 & >= 3.
						unreg = "\""+aParam+"\" (app exit)";
						break;
					case "smDlg":
						unreg = "opening restart in \"Safe Mode\" default dialog";
						break;
					case "ppbm":
						unreg = "enabling/disabling permanent private browsing mode";
						break;
				};
				this.promptSrvc.alert(null, "Restart Request Observer:", "Unregistered" + "\nBy: " + unreg);
			};
		};
    },

	onApplicationStartup: function() {
		if(appVersion >= 4) {
			var ftrPrefState = this.prefBranch.getPrefType("extensions.restartmanager.forced_to_restart");
			if(ftrPrefState !== this.prefBranch.PREF_INVALID && ftrPrefState == this.prefBranch.PREF_STRING) {
				if(JSON.parse(this.rmPrefBranch.getCharPref("forced_to_restart"))[0] === true) {
					this.obsSrvc.addObserver(this, "sessionstore-state-read", false);
					propStoreInternal.forceRestarted = true;
				};
			};
		};
		this.obsSrvc.addObserver(this, "sessionstore-windows-restored", false);
		if(appVersion >= 3.6) {
			this.obsSrvc.addObserver(this, "quit-application-granted", false);
		} else {
			this.obsSrvc.addObserver(this, "quit-application", false);
		};
		if((appVersion >= 3.5) && this.nsIEnv.get("MOZ_NO_REMOTE")) {
			propStoreInternal.startedWithNoRemote = true;
		};
		if(appVersion < 4) {
			return;
		};

		// If we are going to be disabled/uninstalled, remove dev prefs and don't allow "force restart".
		var disableSpecials = function() { // "force restart", dev prefs.
			if(this.rmPrefBranch.getBoolPref("invalidate_caches")) {
				this.rmPrefBranch.setBoolPref("invalidate_caches", false);
				var allBrWinEnum = this.winMed.getEnumerator("navigator:browser");
				while(allBrWinEnum.hasMoreElements()) {
					allBrWinEnum.getNext()._RestMan.locals.invalCaches = false;
				};
			};
			if(this.rmPrefBranch.getBoolPref("invalCaches_highlight")) { // Our toolbar button & menu items.
				var currentBrWindow = this.winMed.getMostRecentWindow("navigator:browser");
				if(currentBrWindow._RestMan) {
					currentBrWindow._RestMan.applyUISettings();
					this.rmPrefBranch.setBoolPref("invalCaches_highlight", false);
				};
			};
			if(this.prefBranch.getBranch("nglayout.").getPrefType("debug.disable_xul_fastload") !== this.prefBranch.PREF_INVALID) {
				this.prefBranch.getBranch("nglayout.").clearUserPref("debug.disable_xul_fastload");
			};
			if(this.prefBranch.getBranch("nglayout.").getPrefType("debug.disable_xul_cache") !== this.prefBranch.PREF_INVALID) {
				this.prefBranch.getBranch("nglayout.").clearUserPref("debug.disable_xul_cache");
			}; // For these two "nglayout." prefs there are no default values. We do "clearUserPref" instead
			   // of set them to "false" on disabling/uninstalling, to have them removed after restart, to not
			   // leave garbage behind us.
			propStoreInternal.allowSpecials = false;
		}.bind(this);
		var addonListeners = {
			onDisabling: function(addon) {
				if(addon.id == "restartmanager@chania.gr") {
					disableSpecials();
				};
			},
			onUninstalling: function(addon) {
				if(addon.id == "restartmanager@chania.gr") {
					disableSpecials();
				};
			},
			onOperationCancelled: function(addon) {
				if(addon.id == "restartmanager@chania.gr") {
					if(addon.pendingOperations !== this.AddonManager.PENDING_DISABLE &&
					   addon.pendingOperations !== this.AddonManager.PENDING_UNINSTALL) {
						propStoreInternal.allowSpecials = true;
					};
				};
			}.bind(this)
		};
		Cu.import("resource://gre/modules/AddonManager.jsm", this);
		this.AddonManager.addAddonListener(addonListeners);
		Object.freeze(this);
		if(this.rmPrefBranch.getBoolPref("invalidate_caches")) {
			this.obsSrvc.notifyObservers(null, "startupcache-invalidate", null);
			this.obsSrvc.notifyObservers(null, "chrome-flush-skin-caches", null);
			this.obsSrvc.notifyObservers(null, "chrome-flush-caches", null);
		};
	},

	restoreSettingsOnQuit: function() {
		if(!propStoreInternal.weAreRestarting && this.rmPrefBranch.getIntPref("restore_UserSettings") !== 0) {
			var value = this.rmPrefBranch.getIntPref("restore_UserSettings");
			if(value == 1) {
				this.brPrefBranch.setIntPref("startup.page", 3);
				this.brPrefBranch.setBoolPref("warnOnQuit", true); // pm's default.
			} else if(value == 2) {
				this.brPrefBranch.setIntPref("startup.page", 3);
			};
			this.rmPrefBranch.setIntPref("restore_UserSettings", 0);
		};
	},

	getPrvBrowsingAutostart: function(onAppStartup) { // AppVer >= 20.
	// It is used only if "codeMonitoring" is enabled, to determine if the alert message of "sessionSaveHelper"
	// can be shown or not, on app exit/restart/close last window, in private browsing mode. If app was started
	// with "privatebrowsing.autostart = true", we want this function to return always "true", because there are
	// no any savable data (non-private) to be saved on exit or restart.
	// If app was started with "privatebrowsing.autostart = false" and this pref is changed to "true" during the
	// session in "about:config" and not in UI, so a restart is not mandatory, we also want this function to return
	// always "true" till the end of the session.
		if(onAppStartup) {
			if(this.brPrefBranch.getBoolPref("privatebrowsing.autostart")) {
				propStoreInternal.allowCheck_prvBrowsingAutostart = false;
			};
			return;
		};
		if(propStoreInternal.allowCheck_prvBrowsingAutostart) {
			if(!this.brPrefBranch.getBoolPref("privatebrowsing.autostart")) {
				return false;
			};
			return true;
		};
		return true;
	}
};

// Component registration.
if(appVersion >= 3) {
	Cu.import("resource://gre/modules/XPCOMUtils.jsm", this);
//	if(appVersion>=20) { // It works, but causes almost quadruple of memory commitment..
//		Cu.import("chrome://restartmanager/content/restmanMain.js");
//	};
	if(this.XPCOMUtils.generateNSGetFactory) {
		this.NSGetFactory = this.XPCOMUtils.generateNSGetFactory([RestartRequestObserver]); // Gecko >= 2.0
		Object.freeze(RestartRequestObserver.prototype);
	} else if(this.XPCOMUtils.generateNSGetModule) {
		this.NSGetModule = this.XPCOMUtils.generateNSGetModule([RestartRequestObserver]); // Gecko = 1.9.x
	};
};

// We call our js component this way:
// Cc["@restart_manager/chania.gr;1"].getService().wrappedJSObject;

/*******************************************************************************************/
